﻿using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// 비트맵 헬퍼
    /// </summary>
    public class BitmapHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 픽셀 비트 수
        /// </summary>
        public const int PIXEL_BIT_COUNT = 32;

        /// <summary>
        /// 이미지 바이트 배열
        /// </summary>
        public byte[] ImageByteArray;

        /// <summary>
        /// 행 바이트 수
        /// </summary>
        public int RowByteCount;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 소스 비트맵
        /// </summary>
        private Bitmap sourceBitmap;

        /// <summary>
        /// 소스 비트맵 데이터
        /// </summary>
        private BitmapData sourceBitmapData;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 너비 - Width

        /// <summary>
        /// 너비
        /// </summary>
        public int Width
        {
            get
            {
                return this.sourceBitmap.Width;
            }
        }

        #endregion
        #region 높이 - Height

        /// <summary>
        /// 높이
        /// </summary>
        public int Height
        {
            get
            {
                return this.sourceBitmap.Height;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - BitmapHelper(sourceBitmap)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="sourceBitmap">소스 비트맵</param>
        public BitmapHelper(Bitmap sourceBitmap)
        {
            this.sourceBitmap = sourceBitmap;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 픽셀 구하기 - GetPixel(x, y, red, green, blue, alpha)

        /// <summary>
        /// 픽셀 구하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="red">적색 채널</param>
        /// <param name="green">녹색 채널</param>
        /// <param name="blue">청색 채널</param>
        /// <param name="alpha">알파 채널</param>
        public void GetPixel(int x, int y, out byte red, out byte green, out byte blue, out byte alpha)
        {
            int i = y * this.sourceBitmapData.Stride + x * 4;

            blue  = ImageByteArray[i++];
            green = ImageByteArray[i++];
            red   = ImageByteArray[i++];
            alpha = ImageByteArray[i  ];
        }

        #endregion
        #region 적색 채널 구하기 - GetRed(x, y)

        /// <summary>
        /// 적색 채널 구하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <returns>적색 채널</returns>
        public byte GetRed(int x, int y)
        {
            int i = y * this.sourceBitmapData.Stride + x * 4;

            return ImageByteArray[i + 2];
        }

        #endregion
        #region 녹색 채널 구하기 - GetGreen(x, y)

        /// <summary>
        /// 녹색 채널 구하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <returns>녹색 채널</returns>
        public byte GetGreen(int x, int y)
        {
            int i = y * this.sourceBitmapData.Stride + x * 4;

            return ImageByteArray[i + 1];
        }

        #endregion
        #region 청색 채널 구하기 - GetBlue(x, y)

        /// <summary>
        /// 청색 채널 구하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <returns>청색 채널</returns>
        public byte GetBlue(int x, int y)
        {
            int i = y * this.sourceBitmapData.Stride + x * 4;

            return ImageByteArray[i];
        }

        #endregion
        #region 알파 채널 구하기 - GetAlpha(x, y)

        /// <summary>
        /// 알파 채널 구하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <returns>알파 채널</returns>
        public byte GetAlpha(int x, int y)
        {
            int i = y * this.sourceBitmapData.Stride + x * 4;

            return ImageByteArray[i + 3];
        }

        #endregion

        #region 픽셀 설정하기 - SetPixel(x, y, red, green, blue, alpha)

        /// <summary>
        /// 픽셀 설정하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="red">적색 채널</param>
        /// <param name="green">녹색 채널</param>
        /// <param name="blue">청색 채널</param>
        /// <param name="alpha">알파 채널</param>
        public void SetPixel(int x, int y, byte red, byte green, byte blue, byte alpha)
        {
            int i = y * this.sourceBitmapData.Stride + x * 4;

            ImageByteArray[i++] = blue;
            ImageByteArray[i++] = green;
            ImageByteArray[i++] = red;
            ImageByteArray[i  ] = alpha;
        }

        #endregion
        #region 적색 채널 설정하기 - SetRed(x, y, red)

        /// <summary>
        /// 적색 채널 설정하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="red">적색 채널</param>
        public void SetRed(int x, int y, byte red)
        {
            int i = y * this.sourceBitmapData.Stride + x * 4;

            ImageByteArray[i + 2] = red;
        }

        #endregion
        #region 녹색 채널 설정하기 - SetGreen(x, y, green)

        /// <summary>
        /// 녹색 채널 설정하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="green">녹색 채널</param>
        public void SetGreen(int x, int y, byte green)
        {
            int i = y * this.sourceBitmapData.Stride + x * 4;

            ImageByteArray[i + 1] = green;
        }

        #endregion
        #region 청색 채널 설정하기 - SetBlue(x, y, blue)

        /// <summary>
        /// 청색 채널 설정하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="blue">청색 채널</param>
        public void SetBlue(int x, int y, byte blue)
        {
            int i = y * this.sourceBitmapData.Stride + x * 4;

            ImageByteArray[i] = blue;
        }

        #endregion
        #region 알파 채널 설정하기 - SetAlpha(x, y, alpha)

        /// <summary>
        /// 알파 채널 설정하기
        /// </summary>
        /// <param name="x">X</param>
        /// <param name="y">Y</param>
        /// <param name="alpha">알파 채널</param>
        public void SetAlpha(int x, int y, byte alpha)
        {
            int i = y * this.sourceBitmapData.Stride + x * 4;

            ImageByteArray[i + 3] = alpha;
        }

        #endregion

        #region 비트맵 잠금 설정하기 - LockBitmap()

        /// <summary>
        /// 비트맵 잠금 설정하기
        /// </summary>
        public void LockBitmap()
        {
            Rectangle sourceRectangle = new Rectangle
            (
                0,
                0,
                this.sourceBitmap.Width,
                this.sourceBitmap.Height
            );

            this.sourceBitmapData = this.sourceBitmap.LockBits
            (
                sourceRectangle,
                ImageLockMode.ReadWrite,
                PixelFormat.Format32bppArgb
            );

            RowByteCount = this.sourceBitmapData.Stride;

            int totalByteCount = this.sourceBitmapData.Stride * this.sourceBitmapData.Height;

            ImageByteArray = new byte[totalByteCount];

            Marshal.Copy(this.sourceBitmapData.Scan0, ImageByteArray, 0, totalByteCount);
        }

        #endregion
        #region 비트맵 잠금 해제하기 - UnlockBitmap()

        /// <summary>
        /// 비트맵 잠금 해제하기
        /// </summary>
        public void UnlockBitmap()
        {
            int totalByteCount = this.sourceBitmapData.Stride * this.sourceBitmapData.Height;

            Marshal.Copy(ImageByteArray, 0, this.sourceBitmapData.Scan0, totalByteCount);

            this.sourceBitmap.UnlockBits(this.sourceBitmapData);

            ImageByteArray = null;

            this.sourceBitmapData = null;
        }

        #endregion
    }
}