﻿using System.Drawing;

namespace TestProject
{
    /// <summary>
    /// 그래픽스 확장
    /// </summary>
    public static class GraphicsExtension
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 사각형 그리기 - DrawRectangle(graphics, pen, rectangle)

        /// <summary>
        /// 사각형 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="pen">펜</param>
        /// <param name="rectangle">사각형</param>
        public static void DrawRectangle(this Graphics graphics, Pen pen, RectangleF rectangle)
        {
            graphics.DrawRectangle(pen, rectangle.X, rectangle.Y, rectangle.Width, rectangle.Height);
        }

        #endregion
        #region 상자 그리기 - DrawBox(graphics, brush, pen, centerPoint, radius)

        /// <summary>
        /// 상자 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="brush">브러시</param>
        /// <param name="pen">펜</param>
        /// <param name="centerPoint">중심 포인트</param>
        /// <param name="radius">반경</param>
        public static void DrawBox(this Graphics graphics, Brush brush, Pen pen, PointF centerPoint, float radius)
        {
            RectangleF rectangle = new RectangleF
            (
                centerPoint.X - radius,
                centerPoint.Y - radius,
                2 * radius,
                2 * radius
            );

            graphics.FillRectangle(brush, rectangle);

            graphics.DrawRectangle(pen, rectangle);
        }

        #endregion
        #region 이미지 확대/축소하기 - ScaleImage(sourceBitmap, scale)

        /// <summary>
        /// 이미지 확대/축소하기
        /// </summary>
        /// <param name="sourceBitmap">소스 비트맵</param>
        /// <param name="scale">스케일</param>
        /// <returns>비트맵</returns>
        public static Bitmap ScaleImage(this Bitmap sourceBitmap, float scale)
        {
            if(sourceBitmap == null)
            {
                return null;
            }

            int widthScaled  = (int)(sourceBitmap.Width  * scale);
            int heightScaled = (int)(sourceBitmap.Height * scale);

            Bitmap targetBitmap = new Bitmap(widthScaled, heightScaled);

            using(Graphics graphics = Graphics.FromImage(targetBitmap))
            {
                Point[] targetPointArray =
                {
                    new Point(0              , 0               ),
                    new Point(widthScaled - 1, 0               ),
                    new Point(0              , heightScaled - 1)
                };

                Rectangle targetRectangle = new Rectangle
                (
                    0,
                    0,
                    sourceBitmap.Width  - 1,
                    sourceBitmap.Height - 1
                );

                graphics.DrawImage(sourceBitmap, targetPointArray, targetRectangle, GraphicsUnit.Pixel);
            }

            return targetBitmap;
        }

        #endregion
    }
}